/**
  *  \file game/bbox.h
  *  \brief Starchart Bounding Box
  *
  *  (c) 2001-2013 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_BBOX_H
#define PCC_V2_GAME_BBOX_H

#include "game/coord.h"         // GPoint, chart_conf

class GUniverse;
class GDrawing;

//! Starchart Bounding Box.
class GChartBBox {
 public:
    enum {
        MAX_ZOOM = 10
    };

    void initFromUniverse(const GUniverse& trn);
    void initMinZoom();
    void initMaxZoom();

    void addPoint(GPoint pt);
    void addCircle(GPoint pt, int radius);
    void addDrawing(const GDrawing& d);

    int getMinZoom() const;
    int getMaxZoom() const;
    GPoint getMinimumXY() const;
    GPoint getMaximumXY() const;

 private:
    GPoint min, max;
    int    max_zoom;
    int    min_zoom;
};

/** Get minimum allowed zoom level. This is a soft limit, and decides at what
    point we switch from a scrollable starchart to a fixed one. */
inline int
GChartBBox::getMinZoom() const
{
    return min_zoom;
}

/** Get maximum allowed zoom level. This is the maximum zoom level at which
    there is no overflow in coordinates. */
inline int
GChartBBox::getMaxZoom() const
{
    return max_zoom;
}

/** Get minimum used X/Y. */
inline GPoint
GChartBBox::getMinimumXY() const
{
    return min;
}

/** Get maximum used X/Y. */
inline GPoint
GChartBBox::getMaximumXY() const
{
    return max;
}

#endif
