/**
  *  \file gfx/multiclip.h
  *
  *  (c) 2001-2013 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GFX_MULTICLIP_H
#define PCC_V2_GFX_MULTICLIP_H

#include "gfx/gfxutil.h"
#include "gfx/gfx.h"

class GfxMultiClip : public GfxFilter {
    GfxRectSet rect;
 public:
    GfxMultiClip(GfxCanvas& parent);

    void drawHLine(const GfxPoint& pt, int npix, uint32 color, uint8 pat, alpha_type alpha);
    void drawVLine(const GfxPoint& pt, int npix, uint32 color, uint8 pat, alpha_type alpha);
    void drawPixel(const GfxPoint& pt, uint32 color, alpha_type alpha);
    void drawBar(GfxRect rect, uint32 color, uint32 bg, const GfxFillPattern& pat, alpha_type alpha);

    void blitSurface(const GfxPoint& pt, GfxCanvas& src, GfxRect rect);
    void blitPattern(GfxRect rect, const GfxPoint& pt, int bytes_per_line, const uint8* data, uint32 color, uint32 bg, alpha_type alpha);

    GfxRect computeClipRect(GfxRect r);
    bool isClipped(GfxRect r);
    bool isVisible(GfxRect r);

    void add(const GfxRect& r);
    void remove(const GfxRect& r);
    void clipRegionAtRectangle(const GfxRect& r);
    void clear() { rect.clear(); }
    bool empty() const { return rect.empty(); }

    const GfxRectSet& getRectSet() const { return rect; }
};

#endif
