/**
  *  \file int/if/mineif.h
  *  \brief Interpreter Game Interface: Minefields
  *
  *  (c) 2001-2013 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_IF_MINEIF_H
#define PCC_V2_INT_IF_MINEIF_H

#include "cpluslib/types.h"
#include "int/context.h"
#include "game/mine.h"
#include "int/if/objcontext.h"

class IntValue;
class GMinefield;
class IntArgBlock;

IntValue*   IFMinefieldGet(IntArgBlock& args);
IntContext* IFMinefieldMake();
int32_t     IFMinefieldDim(int32_t dim);

/** Minefield context. */
class IntMinefieldContext : public IntObjectContext {
 public:
    IntMinefieldContext(int mid);
    ~IntMinefieldContext();

    // IntContext:
    int32_t   lookup(const string_t& name);
    void      set(int32_t index, IntValue* value);
    IntValue* get(int32_t index);
    IntContext* clone();
    string_t  toString(bool readable);
    void      store(IntTagNode& sv, Stream& aux);
    void      enumProperties(IntPropertyAcceptor& acceptor);
};

/** Minefield property identifier. */
enum IntMinefieldProperty {
    impId,
    impLastScan,
    impLocX,
    impLocY,
    impMarked,
    impRadius,
    impScanType,
    impTypeCode,
    impTypeStr,
    impUnits
};

IntValue* getMinefieldProperty(const GMinefield& mf, IntMinefieldProperty imp);
void      setMinefieldProperty(GMinefield& mf, IntMinefieldProperty imp, IntValue* value);

#endif
