/**
  *  \file io/memstream.h
  *
  *  (c) 2001-2013 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_IO_MEMSTREAM_H
#define PCC_V2_IO_MEMSTREAM_H

#include "io/streams.h"
#include "cpluslib/buffer.h"
#include "cpluslib/string.h"

class MemoryStream : public Stream {
    /** Stream content. */
    Buffer<char> data;
    /** File pointer. */
    std::size_t pos;
    /** Name. */
    string_t name;
 public:
    MemoryStream();
    MemoryStream(const string_t& s);
    MemoryStream(const Buffer<char>& buf);
    ~MemoryStream();

    /** Get raw buffer. */
    const Buffer<char>& getBuffer() const
        { return data; }

    void         swapBuffer(Buffer<char>&);
    string_t     getContentsAsString() const;

    std::size_t  read(char* buffer, std::size_t max);
    std::size_t  write(const char* buffer, std::size_t max);
    int          getCapabilities() const;
    void         seek(fpos_t pos);
    fpos_t       getSize();
    fpos_t       getPos();
    string_t     getName();

    void         setName(string_t name);
};

#endif
