/**
  *  \file u/t_game_checksum.cc
  *
  *  (c) 2001-2013 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "u/t_game.h"
#include "game/checksum.h"

static const char someTest[] = "SOMETEST";
static const char someUmlauts[] = "\xE4\xF6\xFC";
static const char someUmlauts2[] = "\0\xE4\xF6\xFC";

/** Test getChecksum(). */
void
GameChecksumTestSuite::testChecksum()
{
    // trivial cases
    TS_ASSERT_EQUALS(getChecksum(0, 0), 0);
    TS_ASSERT_EQUALS(getChecksum("", 0), 0);
    TS_ASSERT_EQUALS(getChecksum("", 1), 0);        // includes trailing null byte

    TS_ASSERT_EQUALS(getChecksum("a", 1), 'a');
    TS_ASSERT_EQUALS(getChecksum("a", 2), 'a');     // includes trailing null byte

    TS_ASSERT_EQUALS(getChecksum("\xff", 1), 255);

    TS_ASSERT_EQUALS(getChecksum(someUmlauts, 3), 726);

    // 83 + 79 + 77 + 69 + 84 + 69 + 83 + 84 = 628
    TS_ASSERT_EQUALS(getChecksum(someTest, 8), 628);
    TS_ASSERT_EQUALS(getChecksum(someTest, 0) + getChecksum(someTest+0, 8), 628);
    TS_ASSERT_EQUALS(getChecksum(someTest, 1) + getChecksum(someTest+1, 7), 628);
    TS_ASSERT_EQUALS(getChecksum(someTest, 2) + getChecksum(someTest+2, 6), 628);
    TS_ASSERT_EQUALS(getChecksum(someTest, 3) + getChecksum(someTest+3, 5), 628);
    TS_ASSERT_EQUALS(getChecksum(someTest, 4) + getChecksum(someTest+4, 4), 628);
    TS_ASSERT_EQUALS(getChecksum(someTest, 5) + getChecksum(someTest+5, 3), 628);
    TS_ASSERT_EQUALS(getChecksum(someTest, 6) + getChecksum(someTest+6, 2), 628);
    TS_ASSERT_EQUALS(getChecksum(someTest, 7) + getChecksum(someTest+7, 1), 628);
    TS_ASSERT_EQUALS(getChecksum(someTest, 8) + getChecksum(someTest+8, 0), 628);

    // getChecksum(u16)
    TS_ASSERT_EQUALS(getChecksum(0), 0);
    TS_ASSERT_EQUALS(getChecksum(1), 1);
    TS_ASSERT_EQUALS(getChecksum(42), 42);
    TS_ASSERT_EQUALS(getChecksum(256), 1);
    TS_ASSERT_EQUALS(getChecksum(500), 245);
    TS_ASSERT_EQUALS(getChecksum(65535), 510);
}

/** Test getDigest(). Since these are too hard to compute manually, I generated
    the test digests using PHost's implementation of getDigest(). */
void
GameChecksumTestSuite::testDigest()
{
    // trivial cases
    TS_ASSERT_EQUALS(getDigest(0, 0, 0), 0);
    TS_ASSERT_EQUALS(getDigest(0, 0, 42), 42);

    // less trivial cases. For simplicity, 
    TS_ASSERT_EQUALS(getDigest(someTest, 8), 0x5934F883);
    TS_ASSERT_EQUALS(getDigest(someUmlauts, 3), 0x2A39D50F);
    TS_ASSERT_EQUALS(getDigest(someUmlauts, 4), 0x7AE64E40);  // trailing null actualy modifies digest
    TS_ASSERT_EQUALS(getDigest(someUmlauts2, 4), 0x0B47A972); // leading null as well

    // distributive law (inner call is start of data)
    TS_ASSERT_EQUALS(getDigest(someTest+0, 8, getDigest(someTest, 0)), 0x5934F883);
    TS_ASSERT_EQUALS(getDigest(someTest+1, 7, getDigest(someTest, 1)), 0x5934F883);
    TS_ASSERT_EQUALS(getDigest(someTest+2, 6, getDigest(someTest, 2)), 0x5934F883);
    TS_ASSERT_EQUALS(getDigest(someTest+3, 5, getDigest(someTest, 3)), 0x5934F883);
    TS_ASSERT_EQUALS(getDigest(someTest+4, 4, getDigest(someTest, 4)), 0x5934F883);
    TS_ASSERT_EQUALS(getDigest(someTest+5, 3, getDigest(someTest, 5)), 0x5934F883);
    TS_ASSERT_EQUALS(getDigest(someTest+6, 2, getDigest(someTest, 6)), 0x5934F883);
    TS_ASSERT_EQUALS(getDigest(someTest+7, 1, getDigest(someTest, 7)), 0x5934F883);
    TS_ASSERT_EQUALS(getDigest(someTest+8, 0, getDigest(someTest, 8)), 0x5934F883);
}
